#include <engine/2d/textbox.h>

/*
 * textbox_print:
 *	Prints all SPRTs of TextBox onto the screen.
 */
void
textbox_print(p_box)
struct TextBox *p_box;
{
	int i;
	int row, col;

	int offset_x = 0, offset_y = 0;
	int message_length = strlen(p_box->data.message);

	/*
	 * Since offset_x gets reset to 0 everytime we get a newline,
	 * we record the furthest right position offset_x has gone with
	 * debug_right_limit to set our debug members with.
	 */
	int debug_right_limit = 0;

	/* set TPAGE for SPRTs in the for loop */
	DrawPrim(&p_box->dr_tpage);

	for(i = 0; i < message_length; i++)
	{
		/* perform newline + carriage return if we find '\n' */
		if(p_box->data.message[i] == '\n')
		{
			offset_x = 0;
			offset_y += 2;
			continue;
		}

		if(p_box->data.message[i] == ' ')
		{
			offset_x += 1;
			continue;
		}

		/*
		 * Convert the ASCII values present in message data to the
		 * corresponding row and column in the font texture. This
		 * helps determine the UV values for each SPRT in the
		 * data.display array.
		 */

		col = (p_box->data.message[i] - 33)%10;
		row = (p_box->data.message[i] - 33)/10;

		/* determine UV values for each SPRT */
		p_box->data.display[i].u0 = p_box->u + (col * p_box->char_w);
		p_box->data.display[i].v0 = p_box->v + (row * p_box->char_h);

		/* perform newline + carriage return if we've reached max characters horizontally */
		if(offset_x > p_box->max_x)
		{
			offset_x = 0;
			offset_y += 1;
		}

		/* apply current char position */
		p_box->data.display[i].x0 = ((p_box->x)>>12) + offset_x * p_box->char_w;
		p_box->data.display[i].y0 = ((p_box->y)>>12) + offset_y * p_box->char_h;

		offset_x += 1;

		/* draw */
		DrawPrim(&p_box->data.display[i]);

		/* update debug_right_limit */
		if(debug_right_limit < offset_x)
		{
			debug_right_limit = offset_x;
		}
	}

	if(p_box->is_debug)
	{
		/* update debug box size */
		{
			int left	= (p_box->x)>>12;
			int right	= left + debug_right_limit * p_box->char_w;

			int top		= (p_box->y)>>12;
			int bottom	= top + (offset_y + 1) * p_box->char_h;

			setXY2(&p_box->debug_left, left, top, left, bottom);

			setXY4(&p_box->debug_right,	left, top,
							right, top,
							right, bottom,
							left, bottom);
		}

		/* draw */
		DrawPrim(&p_box->debug_left);
		DrawPrim(&p_box->debug_right);
	}
}

/*
 * textbox_init:
 *	Initializes all members of a TextBox struct.
 */
void
textbox_init(p_box, x, y, u, v, char_w, char_h, max_x, clut_id, tpage_id, is_debug)
struct TextBox *p_box;
int x, y;
unsigned char u, v, char_w, char_h;
unsigned short max_x, clut_id, tpage_id;
int is_debug;
{
	int i;

	p_box->x = x;
	p_box->y = y;
	p_box->u = u;
	p_box->v = v;
	p_box->char_w = char_w;
	p_box->char_h = char_h;
	p_box->max_x = max_x;

	/*
	 * initialize display SPRTs. The u0, v0, x0, and y0 members
	 * are set in print_textbox(), not here.
	 */
	for(i = 0; i < TEXTBOX_MESSAGE_LIMIT; i++)
	{
		SetSprt(&p_box->data.display[i]);

		p_box->data.display[i].w = p_box->char_w; /* should be an even number */
		p_box->data.display[i].h = p_box->char_h;

		p_box->data.display[i].clut = clut_id;

		p_box->data.display[i].r0 = 128;
		p_box->data.display[i].g0 = 128;
		p_box->data.display[i].b0 = 128;
	}

	SetDrawTPage(&p_box->dr_tpage, 0, 0, tpage_id);

	p_box->is_debug = is_debug;

	/* initialize debug members */
	SetLineF2(&p_box->debug_left);
	SetLineF4(&p_box->debug_right);

	setRGB0(&p_box->debug_left, 64, 0, 0);
	setRGB0(&p_box->debug_right, 255, 255, 255);
}

/*
 * textbox_set_color:
 *	Sets the color each SPRT in a TextBox struct.
 */
void
textbox_set_color(p_box, r, g, b)
struct TextBox *p_box;
unsigned char r, g, b;
{
	int i;

	for(i = 0; i < TEXTBOX_MESSAGE_LIMIT; i++)
	{
		setRGB0(&p_box->data.display[i], r, g, b);
	}
}

